/*
 * Copyright (c) 2018 Hai Zhang <dreaming.in.code.zh@gmail.com>
 * All Rights Reserved.
 */

package me.zhanghai.android.files.settings;

import me.zhanghai.android.files.R;
import me.zhanghai.android.files.filelist.FileSortOptions;
import me.zhanghai.android.files.filelist.OpenApkDefaultAction;
import me.zhanghai.android.files.provider.root.RootStrategy;
import me.zhanghai.android.files.settings.SettingsEntries.BooleanSettingsEntry;
import me.zhanghai.android.files.settings.SettingsEntries.EnumSettingsEntry;
import me.zhanghai.android.files.settings.SettingsEntries.ResourceIdSettingsEntry;
import me.zhanghai.android.files.settings.SettingsEntries.StringSettingsEntry;
import me.zhanghai.android.files.settings.SettingsEntries.TypedListSettingsEntry;
import me.zhanghai.android.files.theme.custom.CustomThemeColors;
import me.zhanghai.android.files.theme.night.NightMode;

interface Settings {

    BooleanSettingsEntry FILE_LIST_PERSISTENT_DRAWER_OPEN = new BooleanSettingsEntry(
            R.string.pref_key_file_list_persistent_drawer_open,
            R.bool.pref_default_value_file_list_persistent_drawer_open);

    BooleanSettingsEntry FILE_LIST_SHOW_HIDDEN_FILES = new BooleanSettingsEntry(
            R.string.pref_key_file_list_show_hidden_files,
            R.bool.pref_default_value_file_list_show_hidden_files);

    EnumSettingsEntry<FileSortOptions.By> FILE_LIST_SORT_BY = new EnumSettingsEntry<>(
            R.string.pref_key_file_list_sort_by, R.string.pref_default_value_file_list_sort_by,
            FileSortOptions.By.class);

    EnumSettingsEntry<FileSortOptions.Order> FILE_LIST_SORT_ORDER = new EnumSettingsEntry<>(
            R.string.pref_key_file_list_sort_order,
            R.string.pref_default_value_file_list_sort_order, FileSortOptions.Order.class);

    BooleanSettingsEntry FILE_LIST_SORT_DIRECTORIES_FIRST = new BooleanSettingsEntry(
            R.string.pref_key_file_list_sort_directories_first,
            R.bool.pref_default_value_file_list_sort_directories_first);

    SettingsEntries.ResourceIdSettingsEntry CREATE_ARCHIVE_TYPE = new ResourceIdSettingsEntry(
            R.string.pref_key_create_archive_type, R.string.pref_default_value_create_archive_type);

    EnumSettingsEntry<CustomThemeColors.Primary> PRIMARY_COLOR = new EnumSettingsEntry<>(
            R.string.pref_key_primary_color, R.string.pref_default_value_primary_color,
            CustomThemeColors.Primary.class);

    EnumSettingsEntry<CustomThemeColors.Accent> ACCENT_COLOR = new EnumSettingsEntry<>(
            R.string.pref_key_accent_color, R.string.pref_default_value_accent_color,
            CustomThemeColors.Accent.class);

    EnumSettingsEntry<NightMode> NIGHT_MODE = new EnumSettingsEntry<>(R.string.pref_key_night_mode,
            R.string.pref_default_value_night_mode, NightMode.class);

    BooleanSettingsEntry FILE_LIST_ANIMATION = new BooleanSettingsEntry(
            R.string.pref_key_file_list_animation, R.bool.pref_default_value_file_list_animation);

    TypedListSettingsEntry<StandardDirectorySettings> STANDARD_DIRECTORY_SETTINGS =
            new TypedListSettingsEntry<>(R.string.pref_key_standard_directories,
                    StandardDirectorySettings.CREATOR);

    EnumSettingsEntry<RootStrategy> ROOT_STRATEGY = new EnumSettingsEntry<>(
            R.string.pref_key_root_strategy, R.string.pref_default_value_root_strategy,
            RootStrategy.class);

    StringSettingsEntry ARCHIVE_FILE_NAME_ENCODING = new StringSettingsEntry(
            R.string.pref_key_archive_file_name_encoding,
            R.string.pref_default_value_archive_file_name_encoding);

    EnumSettingsEntry<OpenApkDefaultAction> OPEN_APK_DEFAULT_ACTION = new EnumSettingsEntry<>(
            R.string.pref_key_open_apk_default_action,
            R.string.pref_default_value_open_apk_default_action, OpenApkDefaultAction.class);
}
